/* Copyright (c) 2007 Google Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

//
//  GDataFeedTest.m
//

#import "GData.h"

#import "GDataFeedTest.h"
#import "GDataEntryCalendarEvent.h"
#import "GDataElementsTest.h"
#import "GDataEntryYouTubeVideo.h"

@implementation GDataFeedTest

- (void)runTests:(TestKeyPathValues *)tests {
  
  // step through each feed test
  for (int testIndex = 0;
       tests[testIndex].str1 != nil;
       testIndex++) {
    
    // get the class of this feed and the path to the test file of xml data 
    NSString *className = tests[testIndex].str1;
    Class gdataClass = NSClassFromString(className);
    STAssertNotNil(gdataClass, @"Cannot make class for class name: %@", className);
    
    NSString *feedPath = tests[testIndex].str2;
    NSData *data = [NSData dataWithContentsOfFile:feedPath];
    STAssertNotNil(data, @"Cannot read feed from %@", feedPath);
    
    // create the feed object
    GDataFeedBase *feed1 = nil;
    if ([gdataClass instancesRespondToSelector:@selector(initWithData:)]) {
      feed1 = [[[gdataClass alloc] initWithData:data] autorelease];
    } else {
      // this "feed" isn't a proper feed, and might be an entry
      NSError *error = nil;
      NSXMLDocument *doc = [[[NSXMLDocument alloc] initWithData:data
                                                        options:0
                                                          error:&error] autorelease];
      STAssertNotNil(doc, @"Cannot allocate XML document, error %@", error);

      NSXMLElement* root = [doc rootElement];
      feed1 = [[[gdataClass alloc] initWithXMLElement:root 
                                               parent:nil] autorelease];
    }
    
    // copy the feed object 
    GDataObject *feed1copy = [[feed1 copy] autorelease];
    STAssertTrue([feed1 isEqual:feed1copy], @"Failed copy feed from %@ to %@",
                 feed1, feed1copy);
    
    // make a new feed object we'll test against from XML generated by the copy
    NSXMLElement *outputXML = [feed1copy XMLElement];
        
    GDataFeedBase *feed2 = [[[gdataClass alloc] initWithXMLElement:outputXML
                                                            parent:nil] autorelease];

    STAssertTrue([feed2 isEqual:feed1copy], @"Failed using XML \n  %@\n\nto convert\n  %@ \nto\n  %@",  
                outputXML, feed1copy, feed2);
    
    // step through all the key-value path tests
    while (1) {
      
      ++testIndex;
      
      NSString *keyPath = tests[testIndex].str1;
      NSString *expectedValue = tests[testIndex].str2;
      
      if (keyPath == nil || [keyPath length] == 0) break;
      
#if GDATA_USES_LIBXML
      // skip the XMLStrings until we can normalize whitespace and closing
      // brackets and other minor differences
      if ([keyPath hasSuffix:@".XMLString"]) continue;
#endif
      
      NSString *result = [GDataElementsTest valueInObject:feed2
                                forKeyPathIncludingArrays:keyPath];
      
      // if the result wasn't a string but responds to stringValue, then
      // invoke that to get a string
      if ([expectedValue isKindOfClass:[NSString class]]
          && ![result isKindOfClass:[NSString class]]
          && [result respondsToSelector:@selector(stringValue)]) {
        
        result = [(id)result stringValue];     
      }
      
      // we'll test for equality unless the expected result begins "hasPrefix:"
      // or "contains:"
      if ([expectedValue hasPrefix:@"hasPrefix:"]) {
        NSString *prefix = [expectedValue substringFromIndex:[@"hasPrefix:" length]];
        STAssertTrue([result hasPrefix:prefix], @"failed object %@ \n testing key path '%@' for prefix:\n %@ \n!= prefix:\n %@", 
                     feed2, keyPath, result, prefix);  
        
      } else if ([expectedValue hasPrefix:@"contains:"]) {
        
        NSString *substring = [expectedValue substringFromIndex:[@"contains:" length]];
        NSRange range = [result rangeOfString:substring];
        STAssertTrue(range.location != NSNotFound, @"failed object %@ \n testing key path '%@' for substring:\n %@ \n!= contains:\n %@", 
                     feed2, keyPath, result, substring);      
      } else {
        STAssertTrue(AreEqualOrBothNil(result, expectedValue), @"failed object %@ \n testing key path '%@'\n %@ \n!= \n %@", 
                     feed2, keyPath, result, expectedValue);      
      }
    }
  }  
}


- (void)testCalendarFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Calendar Feed
    //
    { @"GDataFeedCalendar", @"Tests/FeedCalendarTest1.xml" },
      
    // GDataFeedCalendar paths 
    { @"title", @"Fred Flintstone's Calendar List" },
    { @"links.1.rel", kGDataLinkRelPost },
    { @"links.2.rel", @"self" },
    { @"authors.0.name", @"Fred Flintstone" },
    { @"authors.0.email", @"fred@gmail.com" },
    { @"generator.URI", @"http://www.google.com/calendar" },
    { @"generator.name", @"Google Calendar" },
    { @"generator.version", @"1.0" },
    { @"startIndex", @"1" },
    { @"itemsPerPage", @"3" },
      
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    // GDataEntryCalendar paths
    { @"entries.0.identifier", @"http://www.google.com/calendar/feeds/test%40domain.net/test%40domain.net" },
    { @"entries.0.publishedDate.RFC3339String", @"2006-11-14T00:03:38Z" },
    { @"entries.0.updatedDate.RFC3339String", @"2006-11-09T00:16:10Z" },
    { @"entries.0.editedDate.RFC3339String", @"2006-11-09T00:16:15Z" },
    { @"entries.0.title", @"Fred Flintstone" },
    { @"entries.0.links.0.rel", @"alternate" },
    { @"entries.0.links.1.href", @"http://www.google.com/calendar/feeds/test%40domain.net/test%40domain.net" },
    { @"entries.0.authors.0.name", @"Fred Flintstone" },
    { @"entries.0.authors.0.email", @"fred@gmail.com" },
    { @"entries.0.isHidden", @"0" },
    { @"entries.0.timeZoneName", @"America/Los_Angeles" },
    { @"entries.0.color", @"#B1365F" },
    { @"entries.0.accessLevel", kGDataCalendarAccessOwner},
    { @"entries.0.overrideName", @"over-ride-name" },
    { @"entries.1.locations.0", @"Joes Pub" },
    { @"entries.2.isSelected", @"0" },
    { @"entries.2.isHidden", @"1" },
    
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed
    
    
    //
    // CalendarEntries Feed
    //
    { @"GDataFeedCalendarEvent", @"Tests/FeedCalendarEventTest1.xml" },
    
    // GDataFeedCalendarEvent paths
    { @"title", @"Fred Flintstone" },
    { @"subtitle", @"Fred Flintstone" },
    { @"links.0.rel", kGDataLinkRelFeed },
    { @"links.2.rel", @"self" },
    { @"authors.0.name", @"Fred Flintstone" },
    { @"authors.0.email", @"fred@gmail.com" },
    { @"identifier", @"http://www.google.com/calendar/feeds/test%40gmail.com/private/full" },
    { @"namespaces.gCal", kGDataNamespaceGCal },
    
    { @"generator.URI", @"http://www.google.com/calendar" },
    { @"generator.name", @"Google Calendar" },
    { @"generator.version", @"1.0" },
    { @"startIndex", @"1" },
    { @"itemsPerPage", @"100000" },
    { @"timeZoneName", @"America/Los_Angeles" },
    
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
    
    // GDataEntryCalendarEvent paths
    { @"entries.0.identifier", @"contains:i12d4avieju0vogcga72aj3908" },
    
    { @"entries.0.publishedDate.RFC3339String", @"2006-10-27T22:48:14Z" },
    { @"entries.0.updatedDate.RFC3339String", @"2006-11-03T21:17:40Z" },
    { @"entries.0.title", @"3 days" },
    { @"entries.0.content", @"The description field" },

    { @"entries.0.links.0.title", @"alternate" },
    { @"entries.0.links.1.rel", @"self" },
    { @"entries.0.authors.0.name", @"Fred Flintstone" },
    { @"entries.0.authors.0.email", @"fred@gmail.com" },
    { @"entries.0.visibility", kGDataEventVisibilityDefault },
    { @"entries.0.comment.feedLink.href", @"contains:i12d4avieju0vogcga72aj3908/comments" },
    { @"entries.0.shouldSendEventNotifications", @"0" },
    { @"entries.0.isQuickAdd", @"0" },
    { @"entries.0.transparency", kGDataEventTransparencyOpaque },
    { @"entries.0.eventStatus", kGDataEventStatusConfirmed },
    { @"entries.0.participants.0.email", @"FredFlintstone@gmail.com" },
    { @"entries.0.participants.0.rel", kGDataWhoEventAttendee },
    { @"entries.0.participants.0.attendeeStatus", kGDataWhoAttendeeStatusDeclined },
    { @"entries.0.participants.1.email", @"FredFlintstone@google.com" },
    { @"entries.0.participants.2.email", @"freg@gmail.com" },
    { @"entries.0.times.0.endTime.RFC3339String", @"2006-11-16" },
    { @"entries.0.times.0.reminders.0.minutes", @"10" },
    { @"entries.0.locations.0", @"The-where-field" },
    { @"entries.0.locations.0.rel", nil },
    { @"entries.0.sequenceNumber", @"2" },
    { @"entries.0.iCalUID", @"4A24A0FF-EA3A-4839-AA09-F4283CB6D345" },
    { @"entries.1.recurrence", @"hasPrefix:DTSTART;VALUE=DATE:20061120" },
    { @"entries.1.reminders.0.minutes", @"10" },
    { @"entries.1.isDeleted", @"0" },
    { @"entries.3.locations.0", @"Seattle" },
    { @"entries.3.isDeleted", @"1" },
        
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
    
    { @"", @"" }, // end of feed

    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}

- (void)testContactsFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Contact Feed
    //
    { @"GDataFeedContact", @"Tests/FeedContactTest1.xml" },
      
    // GDataFeedContact paths 
    { @"title", @"Contacts" },
    { @"categories.0.term", kGDataCategoryContact },
    { @"links.1.rel", kGDataLinkRelPost },
      
    { @"unknownAttributes.@count.stringValue", @"0" },
    { @"unknownChildren.@count.stringValue", @"0" },
      
    // GDataEntryContact paths
    // First entry is real; second entry is deleted
    { @"entries.0.identifier", @"contains:9cfaae9" },
    { @"entries.0.categories.0.term", kGDataCategoryContact },
    { @"entries.0.isDeleted", @"0" },
      
    { @"entries.0.primaryOrganization.orgName", @"Le Company" },
      
    { @"entries.0.organizations.0.orgName", @"Le Company" },
    { @"entries.0.organizations.0.orgTitle", @"Titularstuff" },
    { @"entries.0.organizations.0.label", nil },
    { @"entries.0.organizations.0.rel", kGDataContactOther },
    { @"entries.0.organizations.0.isPrimary", @"1" },
      
    { @"entries.0.organizations.1.orgName", @"Deadhead Associates" },
    { @"entries.0.organizations.1.orgTitle", @"Groupie" },
    { @"entries.0.organizations.1.label", @"DAz" },
    { @"entries.0.organizations.1.rel", nil },
    { @"entries.0.organizations.1.isPrimary", @"0" },
      
    { @"entries.0.primaryIMAddress.address", @"fooaimz" },
      
    { @"entries.0.IMAddresses.0.protocol", kGDataIMProtocolAIM },
    { @"entries.0.IMAddresses.0.address", @"fooaimz" },
    { @"entries.0.IMAddresses.0.label", @"werkz" },
    { @"entries.0.IMAddresses.0.isPrimary", @"1" },
      
    { @"entries.0.IMAddresses.1.protocol", kGDataIMProtocolMSN },
    { @"entries.0.IMAddresses.1.address", @"foomsn" },
    { @"entries.0.IMAddresses.1.label", nil },
    { @"entries.0.IMAddresses.1.rel", kGDataContactHome },
    { @"entries.0.IMAddresses.1.isPrimary", @"0" },
      
    { @"entries.0.IMAddresses.2.protocol", kGDataIMProtocolGoogleTalk },
    { @"entries.0.IMAddresses.2.address", @"foo@gmail.com" },
    { @"entries.0.IMAddresses.2.label", nil },
    { @"entries.0.IMAddresses.2.rel", kGDataContactOther },
    { @"entries.0.IMAddresses.2.isPrimary", @"0" },
      
    { @"entries.0.IMAddresses.3.protocol", kGDataIMProtocolJabber },
    { @"entries.0.IMAddresses.3.address", @"foo@jabber.org" },
    { @"entries.0.IMAddresses.3.label", @"jabz" },
    { @"entries.0.IMAddresses.3.rel", nil },
    { @"entries.0.IMAddresses.3.isPrimary", @"0" },
      
    { @"entries.0.primaryPhoneNumber.stringValue", @"123-4567" },
      
    { @"entries.0.phoneNumbers.0.stringValue", @"123-4567" },
    { @"entries.0.phoneNumbers.0.label", nil },
    { @"entries.0.phoneNumbers.0.rel", kGDataPhoneNumberMobile },
    { @"entries.0.phoneNumbers.0.isPrimary", @"1" },
      
    { @"entries.0.phoneNumbers.1.stringValue", @"333-1414" },
    { @"entries.0.phoneNumbers.1.label", @"shoefone" },
    { @"entries.0.phoneNumbers.1.rel", nil },
    { @"entries.0.phoneNumbers.1.isPrimary", @"0" },
      
    { @"entries.0.primaryPostalAddress.stringValue", @"123 Lane St" },
      
    { @"entries.0.postalAddresses.0.stringValue", @"123 Lane St" },
    { @"entries.0.postalAddresses.0.label", nil },
    { @"entries.0.postalAddresses.0.rel", kGDataContactHome },
    { @"entries.0.postalAddresses.0.isPrimary", @"1" },
      
    { @"entries.0.primaryEmailAddress.address", @"foo@bar.com" },
      
    { @"entries.0.emailAddresses.0.address", @"foo@bar.com" },
    { @"entries.0.emailAddresses.0.label", nil },
    { @"entries.0.emailAddresses.0.rel", kGDataContactHome },
    { @"entries.0.emailAddresses.0.isPrimary", @"1" },
    
    { @"entries.0.emailAddresses.1.address", @"2@bar.com" },
    { @"entries.0.emailAddresses.1.label", @"norzglie" },
    { @"entries.0.emailAddresses.1.rel", nil },
    { @"entries.0.emailAddresses.1.isPrimary", @"0" },
      
    { @"entries.1.identifier", @"contains:b001135" },
    { @"entries.1.categories.0.term", kGDataCategoryContact },
    { @"entries.1.isDeleted", @"1" },
      
    { @"entries.0.unknownAttributes.@count.stringValue", @"0" },
    { @"entries.0.unknownChildren.@count.stringValue", @"0" },
      
    { @"", @"" }, // end of feed
      
    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}



- (void)testGoogleBaseFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Google Base snippet feed
    //
    { @"GDataFeedGoogleBase", @"Tests/FeedGoogleBaseSnippetTest1.xml" },
      
    // feed paths 
    { @"generator.URI", @"http://base.google.com" },
    { @"generator.name", @"GoogleBase" },
    { @"title", @"Items matching query: digital camera" },
    { @"links.0.href", @"http://base.google.com" },
      
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    // entry paths
    // There are three entries; we'll test against the first one
    { @"entries.0.identifier", @"http://www.google.com/base/feeds/snippets/13246453826751927533" },
    { @"entries.0.publishedDate.RFC3339String", @"2007-01-31T02:36:40Z" },
    { @"entries.0.categories.0.scheme", kGDataCategoryGoogleBaseItemTypesScheme },
    { @"entries.0.categories.0.term", @"products" },
    
    { @"entries.0.itemType", @"Products" },
    { @"entries.0.expirationDate.RFC3339String", @"2007-03-02T02:36:40Z" },
    { @"entries.0.imageLink", @"hasPrefix:http://base.google.com/base_image?q=http%3A%2F%2Fwww.bhphotovideo.com%2Fimages%2Fitems%2F305668.jpg" },
    { @"entries.0.imageLinks.@count", @"1" },
    { @"entries.0.price", @"34.95 usd" },
    { @"entries.0.location", @"420 9th Ave. 10001" },
    { @"entries.0.paymentNotes", @"PayPal & Bill Me Later credit available online only." },
    { @"entries.0.customerID", @"1172711" },
    
    // feed batch item
    { @"batchOperation.type", @"update" },
    
    // entry batch items
    { @"entries.0.batchID", @"BatchID 1" },
    { @"entries.0.batchOperation.type", @"insert" },
    { @"entries.0.batchStatus.code", @"201" },
    { @"entries.0.batchStatus.reason", @"Created" },
    { @"entries.2.batchInterrupted.reason", @"interruption reason" },
    { @"entries.2.batchInterrupted.successCount", @"10" },
    { @"entries.2.batchInterrupted.errorCount", @"5" },
    { @"entries.2.batchInterrupted.totalCount", @"15" },

    // feed atom pub control item
    { @"atomPubControl.isDraft", @"1" },

    // entry atom pub control items
    { @"entries.0.atomPubControl.isDraft", @"1" },
    { @"entries.1.atomPubControl.isDraft", @"0" },
    { @"entries.2.atomPubControl.isDraft", nil },

    // do some tests of arbitrary attributes through -attributeDictionary
    { @"entries.0.attributeDictionary.manufacturer_id.0.textValue", @"DCB5092" },
    { @"entries.0.attributeDictionary.target_country.0.textValue", @"US" },
    { @"entries.0.attributeDictionary.weight.0.textValue", @"1.0" },
    { @"entries.0.attributeDictionary.condition.0.textValue", @"new" },
    { @"entries.0.attributeDictionary.condition.0.subAttributes.@count", @"1" },
    { @"entries.0.attributeDictionary.condition.0.subAttributes.0.name", @"greg rating" },
    { @"entries.0.attributeDictionary.condition.0.subAttributes.0.textValue", @"89" },

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
      
    //
    // Google Base item type feed
    //
    { @"GDataFeedGoogleBase", @"Tests/FeedGoogleBaseItemTypesTest1.xml" },
      
      // GDataFeedContact paths 
    { @"generator.URI", @"http://base.google.com" },
    { @"generator.name", @"GoogleBase" },
    { @"title", @"Item types for locale en_US" },
    { @"links.0.href", @"http://base.google.com" },
      
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    // GDataEntryGoogleBase paths
    // There are two entries; we'll test against the first one
    { @"entries.0.identifier", @"http://www.google.com/base/feeds/itemtypes/en_US/products" },
    { @"entries.0.categories.0.scheme", kGDataCategoryGoogleBaseItemTypesScheme },
    { @"entries.0.categories.0.term", @"products" },
    { @"entries.0.title", @"products" },
    { @"entries.0.content", @"products" },
      
    { @"entries.0.metadataItemType", @"products" },
    { @"entries.0.metadataAttributeList.attributes.@count", @"67" },
    { @"entries.0.metadataAttributeList.attributes.1.name", @"condition" },
    { @"entries.0.metadataAttributeList.attributes.1.type", @"text" },

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
    
    //
    // Google Base item count feed
    //
    { @"GDataFeedGoogleBase", @"Tests/FeedGoogleBaseItemCountsTest1.xml" },
      
    // GDataFeedContact paths 
    { @"generator.URI", @"http://base.google.com" },
    { @"generator.name", @"GoogleBase" },
    { @"title", @"Attribute histogram for query: digital camera [brand:canon]" },
    { @"links.0.href", @"http://base.google.com" },
      
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    // GDataEntryGoogleBase paths
    // There are two entries; we'll test against the first one
    { @"entries.0.identifier", @"http://www.google.com/attributes/brand%28text%29?bq=digital+camera+%5Bbrand%3Acanon%5D" },
    { @"entries.0.title", @"brand(text)" },
    { @"entries.0.content", @"Attribute \"brand\" of type text in query: digital camera [brand:canon]" },
      
    { @"entries.0.metadataItemType", nil },
    { @"entries.0.metadataAttributes.0.name", @"brand" },
    { @"entries.0.metadataAttributes.0.type", @"text" },
    { @"entries.0.metadataAttributes.0.count", @"133144" },
    { @"entries.0.metadataAttributes.0.values.@count", @"1" },
    { @"entries.0.metadataAttributes.0.values.0.count", @"96775" },
    { @"entries.0.metadataAttributes.0.values.0.contents", @"canon" },
        
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
      
    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}

- (void)testSpreadsheetFeeds {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Spreadsheet feed (list of user's spreadsheets)
    //
    { @"GDataFeedSpreadsheet", @"Tests/FeedSpreadsheetTest1.xml" },
    
    // feed paths
    { @"identifier", @"http://spreadsheets.google.com/feeds/spreadsheets/private/full" },
    { @"links.2.href", @"http://spreadsheets.google.com/feeds/spreadsheets/private/full?tfe=" },
    { @"title", @"Available Spreadsheets - test@foo.net" },
    
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // entry paths
    // There is one entry
    { @"entries.0.identifier", @"http://spreadsheets.google.com/feeds/spreadsheets/private/full/o04181601172097104111.497668944883620000" },
    { @"entries.0.updatedDate.RFC3339String", @"2007-03-22T23:25:53Z" },
    { @"entries.0.categories.2.scheme", kGDataCategorySchemeSpreadsheet },
    { @"entries.0.categories.2.term", kGDataCategorySpreadsheet },
    { @"entries.0.title", @"My Test Spreadsheet" },
    
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
    
    { @"", @"" }, // end of feed
    
    //
    // Worksheet feed (list of a spreadsheet's worksheets)
    //
    { @"GDataFeedWorksheet", @"Tests/FeedSpreadsheetWorksheetTest1.xml" },
      
    // feed paths
    { @"identifier", @"http://spreadsheets.google.com/feeds/worksheets/o04181601172097104111.497668944883620000/private/full" },
    { @"links.2.href", @"http://spreadsheets.google.com/feeds/worksheets/o04181601172097104111.497668944883620000/private/full?tfe=" },
    { @"title", @"My Test Spreadsheet" },
    { @"authors.0.email", @"test@foo.net" },
        
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // entry paths
    // There is one entry
    { @"entries.0.identifier", @"http://spreadsheets.google.com/feeds/worksheets/o04181601172097104111.497668944883620000/private/full/od6" },
    { @"entries.0.updatedDate.RFC3339String", @"2007-03-22T23:28:50Z" },
    { @"entries.0.categories.0.scheme", kGDataCategorySchemeSpreadsheet },
    { @"entries.0.categories.0.term", kGDataCategoryWorksheet },
    { @"entries.0.title", @"Sheet1" },
    { @"entries.0.rowCount", @"100" },
    { @"entries.0.columnCount", @"20" },
    
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
 
    //
    // Cells feed (all of a worksheet's cells)
    //
    { @"GDataFeedSpreadsheetCell", @"Tests/FeedSpreadsheetCellsTest1.xml" },
    
    // feed paths
    { @"identifier", @"http://spreadsheets.google.com/feeds/cells/o04181601172097104111.497668944883620000/od6/private/full" },
    { @"links.0.href", @"http://spreadsheets.google.com/ccc?key=o04181601172097104111.497668944883620000" },
    { @"categories.0.scheme", kGDataCategorySchemeSpreadsheet },
    { @"categories.0.term", kGDataCategorySpreadsheetCell },
    { @"title", @"Sheet1" },
    { @"authors.0.email", @"test@foo.net" },
    { @"rowCount", @"100" },
    { @"columnCount", @"20" },
  
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // entry paths
    // The sheet looks like this (2 cols x 4 rows)
    // Fred	Martha
    // =pi()	    =sin(A2)
    // =1.5*pi()	=sin(A3)
    // =2.0*pi()	=sin(A4)
    
    { @"entries.1.identifier", @"http://spreadsheets.google.com/feeds/cells/o04181601172097104111.497668944883620000/od6/private/full/R1C2" },
    { @"entries.1.updatedDate.RFC3339String", @"2007-03-22T23:28:50Z" },
    { @"entries.1.categories.0.scheme", kGDataCategorySchemeSpreadsheet },
    { @"entries.1.categories.0.term", kGDataCategorySpreadsheetCell },
    { @"entries.1.title", @"B1" },
    { @"entries.1.cell.column", @"2" },
    { @"entries.1.cell.row", @"1" },
    { @"entries.1.cell.inputString", @"Martha" },
    { @"entries.1.cell.numericValue", nil },
    { @"entries.1.cell.resultString", @"Martha" },
    
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    //
    // List feed (all of a worksheet as a list) which contains customElements
    //
    
    // feed paths
    { @"GDataFeedSpreadsheetList", @"Tests/FeedSpreadsheetListTest1.xml" },
    { @"identifier", @"http://spreadsheets.google.com/feeds/list/o04181601172097104111.497668944883620000/od6/private/full" },
    { @"links.0.href", @"http://spreadsheets.google.com/ccc?key=o04181601172097104111.497668944883620000" },
    { @"categories.0.scheme", kGDataCategorySchemeSpreadsheet },
    { @"categories.0.term", kGDataCategorySpreadsheetList },
    { @"title", @"Sheet1" },
    { @"authors.0.email", @"test@foo.net" },
    
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // entry paths
    { @"entries.1.customElementDictionary.fred", @"4.71238898038469" },
    { @"entries.1.customElementDictionary.martha", @"-1" },
    
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed
    
    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}

- (void)testCodeSearchFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Feed of a user's albums
    //
    { @"GDataFeedCodeSearch", @"Tests/FeedCodeSearchTest1.xml" },
      
    // GDataFeedCodeSearch paths
    { @"authors.0.name", @"Google Code Search" },
    { @"authors.0.URI", @"http://www.google.com/codesearch" },

    // GDataEntryCodeSearch paths
    { @"entries.0.package.name", @"http://ftp.funet.fi/pub/CPAN/src/perl-5.9.1.tar.gz" },
    { @"entries.0.package.URI", @"http://ftp.funet.fi/pub/CPAN/src/perl-5.9.1.tar.gz" },

    { @"entries.1.package.name", @"http://gentoo.osuosl.org/distfiles/Perl6-Pugs-6.2.12.tar.gz" },
    { @"entries.1.package.URI", @"http://gentoo.osuosl.org/distfiles/Perl6-Pugs-6.2.12.tar.gz" },
    { @"entries.1.file.name", @"Perl6-Pugs-6.2.12/t/subroutines/sub_named_params.t" },
    { @"entries.1.matches.0.lineNumberString", @"131" },
    { @"entries.1.matches.0.type", @"text/html" },
    { @"entries.1.matches.0", @"hasPrefix:<pre>my %fellowship" },
    { @"entries.1.matches.1.lineNumberString", @"132" },
    { @"entries.1.matches.1.type", @"text/html" },
    { @"entries.1.matches.1", @"hasPrefix:<pre>is(%fellowship&lt;hobbit&gt;" },
      
    { @"", @"" }, // end of feed
      
    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}

- (void)testPhotosFeeds {
  
  // TODO: test geoLocation once we have a good sample of it
  
  // Test a non-ASCII character and some html characters in a TextConstruct.  
  // We'll allocate it dynamically since source code cannot contain non-ASCII.
  NSString *template = @"Test %C Alb%Cm";
  NSString *photoAlbumName = [NSString stringWithFormat:template, 
    0x262F, 0x00FC]; // yin yang, u with umlaut
  
  // Non-ascii photo description, includes the Wheel of Dharma
  NSString *photoDescriptionText = [NSString stringWithFormat:
    @"Caption for the car %C photo", 0x2638];  

  TestKeyPathValues tests[] =
  { 
    //
    // Feed of a user's albums
    //
    { @"GDataFeedPhotoUser", @"Tests/FeedPhotosUserAlbum1.xml" },

    // GDataFeedPhotosAlbum paths
    { @"username", @"TestdomainTestAccount" },
    { @"nickname", @"Greg" },
    { @"thumbnail", @"hasPrefix:http://lh3.google.com/image/TestdomainTestAccount" },
    { @"quotaLimit", @"1073741824" },
    { @"quotaUsed", @"108303" },
    { @"maxPhotosPerAlbum", @"500" },
    { @"categories.0.term", kGDataCategoryPhotosUser },

    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // GDataEntryPhotoAlbum paths
    { @"entries.0.categories.0.term", kGDataCategoryPhotosAlbum },

    { @"entries.0.mediaGroup.mediaTitle", photoAlbumName },
    { @"entries.0.mediaGroup.mediaDescription", @"Album description" },
    { @"entries.0.mediaGroup.mediaCredits.0", @"Greg" },
    { @"entries.0.mediaGroup.mediaContents.0.medium", @"image" },
    { @"entries.0.mediaGroup.mediaContents.0.type", @"image/jpeg" },
    { @"entries.0.mediaGroup.mediaContents.0.URLString", @"hasPrefix:http://lh5.google.com/image/TestdomainTestAccount" },
    { @"entries.0.mediaGroup.mediaThumbnails.0.height", @"160" },
    { @"entries.0.mediaGroup.mediaThumbnails.0.URLString", @"hasPrefix:http://lh5.google.com/image/TestdomainTestAccount" },

    { @"entries.0.GPhotoID", @"5067143575034336993" },
    { @"entries.0.name", @"TestAlbM" },
    { @"entries.0.access", @"public" },
    { @"entries.0.photosUsed", @"2" },
    { @"entries.0.commentCount", @"0" },
    { @"entries.0.bytesUsed", @"108303" },
    { @"entries.0.nickname", @"Greg" },
    { @"entries.0.photosLeft", @"498" },
    { @"entries.0.commentsEnabled", @"1" },
    { @"entries.0.location", @"Album Site" },
    { @"entries.0.timestamp.dateValue.description", @"2007-05-21 00:00:00 -0700" },
    { @"entries.0.username", @"TestdomainTestAccount" },
    { @"entries.0.identifier", @"http://picasaweb.google.com/data/entry/api/user/TestdomainTestAccount/albumid/5067143575034336993" },
    { @"entries.0.title.type", @"text" },
    { @"entries.0.title", photoAlbumName },
    { @"entries.0.photoDescription", @"Album description" },
    { @"entries.0.rightsString", @"public" },

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    //
    // Feed of an album's photos
    //
    { @"GDataFeedPhotoAlbum", @"Tests/FeedPhotosAlbumPhoto1.xml" },

    // GDataFeedPhotoAlbum - feed paths
    { @"GPhotoID", @"5067143575034336993" },
    { @"name", @"TestAlbM" },
    { @"access", @"public" },
    { @"photosUsed", @"2" },
    { @"commentCount", @"0" },
    { @"bytesUsed", @"108303" },
    { @"nickname", @"Greg" },
    { @"photosLeft", @"498" },
    { @"commentsEnabled", @"1" },
    { @"location", @"Album Site" },
    { @"timestamp.dateValue.description", @"2007-05-21 00:00:00 -0700" },
    { @"username", @"TestdomainTestAccount" },
    { @"identifier", @"http://picasaweb.google.com/data/feed/api/user/test%40testdomain.net/albumid/5067143575034336993" },
    { @"title.type", @"text" },
    { @"title", photoAlbumName },
    { @"photoDescription", @"Album description" },
    { @"rights", @"public" },
    { @"categories.0.term", kGDataCategoryPhotosAlbum },

    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // GDataEntryPhoto - entry paths
    { @"entries.0.categories.0.term", kGDataCategoryPhotosPhoto },

    { @"entries.0.position", @"1.1" },
    { @"entries.0.checksum", @"23512309abbs298" },
    { @"entries.0.GPhotoID", @"5067143579329304306" },
    { @"entries.0.version", @"1179786875940336" },
    { @"entries.0.albumID", @"5067143575034336993" },
    { @"entries.0.client", @"Picasa 2.6" },
    { @"entries.0.width", @"660" },
    { @"entries.0.height", @"433" },
    { @"entries.0.commentsEnabled", @"1" },
    { @"entries.0.size", @"87225" },
    { @"entries.0.commentCount", @"1" },
    { @"entries.0.timestamp.dateValue.description", @"2007-05-21 15:25:01 -0700" },
    { @"entries.0.title", @"Car.jpg" },
    { @"entries.0.photoDescription", photoDescriptionText },
    { @"entries.0.content.sourceURI", @"http://lh3.google.com/image/TestdomainTestAccount/RlIcPQ_TFvI/AAAAAAAAAAs/3fvWtQLN3KI/Car.jpg" },
    { @"entries.0.content.type", @"image/jpeg" },

    { @"entries.0.mediaGroup.mediaTitle", @"Car.jpg" },
    { @"entries.0.mediaGroup.mediaDescription", photoDescriptionText },
    { @"entries.0.mediaGroup.mediaCredits.0", @"Greg" },
    { @"entries.0.mediaGroup.mediaContents.0.medium", @"image" },
    { @"entries.0.mediaGroup.mediaContents.0.type", @"image/jpeg" },
    { @"entries.0.mediaGroup.mediaContents.0.URLString", @"http://lh3.google.com/image/TestdomainTestAccount/RlIcPQ_TFvI/AAAAAAAAAAs/3fvWtQLN3KI/Car.jpg" },
    { @"entries.0.mediaGroup.mediaThumbnails.0.height", @"47" },
    { @"entries.0.mediaGroup.mediaThumbnails.0.width", @"72" },
    { @"entries.0.mediaGroup.mediaThumbnails.0.URLString", @"http://lh3.google.com/image/TestdomainTestAccount/RlIcPQ_TFvI/AAAAAAAAAAs/3fvWtQLN3KI/Car.jpg?imgmax=72" },
    { @"entries.0.mediaGroup.mediaKeywords", @"headlight, red car" },

    { @"entries.0.EXIFTags.tagDictionary.exposure", @"0.0080" },
    { @"entries.0.EXIFTags.tagDictionary.imageUniqueID", @"d8a9e8fd57a384d216f4b2a853d654fc" },

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    //
    // Feed of a photo's comments
    //
    { @"GDataFeedPhoto", @"Tests/FeedPhotosPhotoComment1.xml" },

    // GDataFeedPhoto - feed paths
    { @"generator.URI", @"http://picasaweb.google.com/" },
    { @"generator.name", @"Picasaweb" },
    { @"generator.version", @"1.00" },
    { @"EXIFTags.tagDictionary.exposure", @"0.0080" },
    { @"categories.0.term", kGDataCategoryPhotosPhoto },
    { @"EXIFTags.tagDictionary.imageUniqueID", @"d8a9e8fd57a384d216f4b2a853d654fc" },
    { @"position", @"1.1" },
    { @"checksum", @"23512309abbs298" },
    { @"GPhotoID", @"5067143579329304306" },
    { @"version", @"1179786875940336" },
    { @"albumID", @"5067143575034336993" },
    { @"client", @"Picasa 2.6" },
    { @"width", @"660" },
    { @"height", @"433" },
    { @"commentsEnabled", @"1" },
    { @"size", @"87225" },
    { @"commentCount", @"1" },
    { @"timestamp.dateValue.description", @"2007-05-21 15:25:01 -0700" },
    { @"title", @"Car.jpg" },
    { @"photoDescription", photoDescriptionText },
    { @"icon", @"http://lh3.google.com/image/TestdomainTestAccount/RlIcPQ_TFvI/AAAAAAAAAAs/3fvWtQLN3KI/Car.jpg?imgmax=288" },

    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // GDataEntryPhotoComment - entry paths
    { @"entries.0.photoID", @"5067143579329304306" },
    { @"entries.0.GPhotoID", @"5067146044640532244" }, 
    { @"entries.0.categories.0.term", kGDataCategoryPhotosComment },

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    //
    // Feed of a user's tags
    //

    // GDataFeedPhotoUser - feed paths

    { @"GDataFeedPhotoUser", @"Tests/FeedPhotosUserTag1.xml" },
    { @"username", @"TestdomainTestAccount" },
    { @"nickname", @"Greg" },
    { @"thumbnail", @"hasPrefix:http://lh3.google.com/image/TestdomainTestAccount" },
    { @"quotaLimit", @"1073741824" },
    { @"quotaUsed", @"108303" },
    { @"maxPhotosPerAlbum", @"500" },
    { @"categories.0.term", kGDataCategoryPhotosUser },

    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    // GDataEntryPhotoTag - entry paths
    
    { @"entries.0.title", @"headlight" },
    { @"entries.0.photoDescription", @"headlight" }, 
    { @"entries.0.categories.0.term", kGDataCategoryPhotosTag },

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"entries.1.title", @"red car" },
    { @"entries.1.photoDescription", @"red car" }, 
    { @"entries.1.categories.0.term", kGDataCategoryPhotosTag },

    { @"entries.1.unknownAttributes.@count", @"0" },
    { @"entries.1.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    //
    // Feed of a user entry
    //
    // This is really a fake feed created by requesting just a single
    // user entry from the picasa server, using
    // GET http://picasaweb.google.com/data/entry/api/user/<username>
    //
    
    // GDataFeedPhotoUser - feed paths (none)
      
    // GDataEntryPhotoUser - entry paths
    { @"GDataFeedPhotoUser", @"Tests/FeedPhotosUserEntry1.xml" },

    { @"entries.0.nickname", @"Greg" },
    { @"entries.0.username", @"TestdomainTestAccount" }, 
    { @"entries.0.thumbnail", @"hasPrefix:http://lh3.google.com/image/TestdomainTestAccount/AAAAUbcFQeo" },
    { @"entries.0.identifier", @"http://picasaweb.google.com/data/entry/api/user/TestdomainTestAccount" },
    { @"entries.0.categories.0.term", kGDataCategoryPhotosUser },
      
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
    
    { @"", @"" }, // end of feed
      
    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}



- (void)testMessageFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Message Feed
    //
  { @"GDataFeedMessage", @"Tests/FeedMessageTest1.xml" },
    
    // GDataFeedMessage paths
  { @"links.0.href", @"hasPrefix:http://www.google.com/calendar/feeds/default" },
  { @"categories.0.term", kGDataMessage },
    
  { @"unknownAttributes.@count", @"0" },
  { @"unknownChildren.@count", @"0" },
    
    // GDataEntryMessage paths
  { @"entries.0.categories.0.term", kGDataMessage },
  { @"entries.0.categories.1.term", kGDataMessageSent },
  { @"entries.0.identifier", @"http://mymail.example.com/feeds/jo/home/full/e1a2af06df8a563edf9d32ec9fd61e03f7f3b67b" },
  { @"entries.0.content", @"Hi, Fritz -- The information you're looking for is on p. 47." },
  { @"entries.0.title", @"Re: Info?" },
  { @"entries.0.participants.0.rel", kGDataWhoMessageFrom },
  { @"entries.0.participants.1.rel", kGDataWhoMessageTo },
    
  { @"entries.0.unknownAttributes.@count", @"0" },
  { @"entries.0.unknownChildren.@count", @"0" },
    
  { @"", @"" }, // end of feed
    
  { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}


- (void)testDocListFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // Docs Feed
    //
    { @"GDataFeedDocList", @"Tests/FeedDocListTest1.xml" },
    
    { @"identifier", @"http://docs.google.com/feeds/documents/private/full" },

    // Docs Entries.  The elements are all standard atom protocol elements,
    // so we are just interested that the proper entry class has been 
    // instantiated 
    { @"entries.0.className", @"GDataEntryStandardDoc" },
    { @"entries.0.categories.0.term", @"http://schemas.google.com/docs/2007#document"},
    { @"entries.0.categories.0.label", @"document"},

    { @"entries.1.className", @"GDataEntrySpreadsheetDoc" },
    { @"entries.1.categories.0.term", @"http://schemas.google.com/docs/2007#spreadsheet"},
    { @"entries.1.categories.0.label", @"spreadsheet"},
    
    { @"entries.2.className", @"GDataEntryPresentationDoc" },
    { @"entries.2.categories.0.term", @"http://schemas.google.com/docs/2007#presentation"},
    { @"entries.2.categories.0.label", @"presentation"},
  
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
      
      
    { @"", @"" }, // end of feed
      
    { nil, nil } // end of test array
  };
  [self runTests:tests];
};
  
- (void)testYouTubeFeeds {
  
  TestKeyPathValues tests[] =
  { 
    // not that the user profile "feed" is really an entry
    { @"GDataEntryYouTubeUserProfile", @"Tests/FeedYouTubeUserProfile1.xml" },

    // entry elements
    { @"identifier", @"http://gdata.youtube.com/feeds/api/users/TestAccount" },

    // subscription entry elements
    { @"className", @"GDataEntryYouTubeUserProfile" },    

    { @"channelType", @"Standard" },
    { @"statistics.lastWebAccess.RFC3339String", @"2008-01-30T11:38:20-08:00" },
    { @"statistics.videoWatchCount", @"1" },
    { @"statistics.viewCount", @"2" },
    
    { @"books", @"Booksz" },
    { @"age", @"49" },
    { @"thumbnail.URLString", @"http://i.ytimg.com/vi/Bkhnvn1PIHs/default.jpg" },
    { @"company", @"Apple, Google" },
    { @"gender", @"m" },
    { @"hobbies", @"Hobbiez, and more hobbiez" },
    { @"hometown", @"Oakland, CA" },
    { @"location", @"Seattle, WA, US" },
    { @"movies", @"Moviez" },
    { @"music", @"Musicz" },
    { @"occupation", @"worker, player" },
    { @"relationship", @"open" },
    { @"school", @"Wildwood Elementary" },
    { @"username", @"TestAccount" },
    { @"firstName", @"Test" },
    { @"lastName", @"Account" },
    { @"youTubeDescription", @"About me stuff" },
    
    { @"feedLinks.@count", @"6" },

    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    
    // video feed
    { @"GDataFeedYouTubeVideo", @"Tests/FeedYouTubeVideo1.xml" },
    
    // video entry elements
    { @"entries.0.statistics.viewCount", @"177" },
    { @"entries.0.comment.feedLink.href", @"http://gdata.youtube.com/feeds/api/videos/Dl643JFJWig/comments" },
    { @"entries.0.links.0.youTubeToken.stringValue", @"ZeeToken" },
    { @"entries.0.isRacy", @"1" },
    { @"entries.0.isEmbeddable", @"0" },
    { @"entries.0.rating.average", @"3" },
    { @"entries.0.rating.min", @"1" },
    { @"entries.0.rating.max", @"5" },
    { @"entries.0.rating.numberOfRaters", @"4" },
    { @"entries.0.mediaGroup.duration", @"3266" },
    { @"entries.0.mediaGroup.isPrivate", @"1" },
    { @"entries.0.mediaGroup.mediaPlayers.0.URLString", @"http://www.youtube.com/watch?v=Dl643JFJWig" },
    { @"entries.0.publicationState.state", @"rejected" },
    { @"entries.0.publicationState.reasonCode", @"32" },
    { @"entries.0.publicationState.helpURLString", @"http://www.youtube.com/" },
    { @"entries.0.publicationState.errorDescription", @"incorrect format" },

    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
    
    // playlist link feed
    
    { @"GDataFeedYouTubePlaylistLink", @"Tests/FeedYouTubePlaylistLink1.xml" },
    { @"categories.0.term", kGDataCategoryYouTubePlaylistLink },
  
    { @"entries.0.className", @"GDataEntryYouTubePlaylistLink" },    
    { @"entries.0.isPrivate", @"1" },
    { @"entries.0.categories.0.term", kGDataCategoryYouTubePlaylistLink },
    { @"entries.0.categories.1.scheme", kGDataSchemeYouTubeTag },
    { @"entries.0.categories.1.term", @"moreplaylisttags" },
    
    { @"entries.1.categories.0.scheme", kGDataSchemeYouTubeTag },
    { @"entries.1.categories.0.term", @"PlaylistTags" },
    { @"entries.1.categories.1.term", kGDataCategoryYouTubePlaylistLink },
    { @"entries.1.youTubeDescription", @"My Playlist Description" },
    { @"entries.1.isPrivate", @"0" },
    
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
        
    // playlist feed
      
    { @"GDataFeedYouTubePlaylist", @"Tests/FeedYouTubePlaylist1.xml" },
  
    { @"categories.0.term", kGDataCategoryYouTubePlaylist },
    { @"categories.1.scheme", kGDataSchemeYouTubeTag }, 
    { @"categories.1.term", @"PlaylistTags" },
    
    { @"mediaGroup.mediaContents.0.URLString", @"http://www.youtube.com/ep.swf?id=BBA2B78EF2F08B10" }, 
    { @"mediaGroup.mediaContents.0.youTubeFormatNumber", @"5" }, 
    
    { @"entries.0.className", @"GDataEntryYouTubePlaylist" },    
    { @"entries.0.categories.0.term", kGDataCategoryYouTubePlaylist },
    { @"entries.0.mediaGroup.mediaTitle", @"The Graphing Calculator Story" }, 
    { @"entries.0.mediaGroup.mediaDescription", @"Google TechTalks" }, 
    { @"entries.0.mediaGroup.mediaKeywords.keywords.1", @"avitzur" }, 
    { @"entries.0.mediaGroup.duration", @"3266" }, 
    { @"entries.0.statistics.viewCount", @"185" }, 
    { @"entries.0.position", @"1" }, 
      
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed
      
    
    // contacts feed
    { @"GDataFeedYouTubeFriend", @"Tests/FeedYouTubeContacts1.xml" },
      
    { @"categories.0.term", kGDataCategoryYouTubeFriend },

    { @"entries.0.className", @"GDataEntryYouTubeFriend" },    

    { @"entries.0.username", @"fredflintstone" },    
    { @"entries.0.status", @"accepted" },    
      
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },
      
    { @"", @"" }, // end of feed

    // subscription feed
    { @"GDataFeedYouTubeSubscription", @"Tests/FeedYouTubeSubscriptions1.xml" },

    // feed elements
    { @"identifier", @"http://gdata.youtube.com/feeds/api/users/testaccount/subscriptions?start-index=1&max-results=25" },
    { @"logo", @"http://www.youtube.com/img/pic_youtubelogo_123x63.gif" },

    // subscription entry elements
    { @"entries.0.className", @"GDataEntryYouTubeSubscription" },    

    { @"entries.0.subscriptionType", @"channel" },
    { @"entries.0.username", @"googletechtalks" },
    
    { @"entries.1.youTubeQueryString", @"cute dogs" },

    // feed link entry elements
    { @"entries.0.feedLink.rel", kGDataLinkYouTubeUploads },
    // youTubeDescription, thumbnail

    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

    { @"", @"" }, // end of feed

    { nil, nil } // end of test array
  };
  [self runTests:tests];
};

- (void)testACLFeed {
  
  TestKeyPathValues tests[] =
  { 
    //
    // ACL Feed
    //
    { @"GDataFeedACL", @"Tests/FeedACLTest1.xml" },
      
      // GDataFeedACL paths
    { @"links.0.href", @"http://www.google.com/calendar/feeds/test%40gmail.com/private/full" },
    { @"links.0.rel", kGDataLinkRelControlledObject },
    { @"categories.0.term", kGDataCategoryACL },
    { @"categories.0.scheme", kGDataCategoryScheme },
      
    { @"unknownAttributes.@count", @"0" },
    { @"unknownChildren.@count", @"0" },
      
      // GDataEntryACL paths (scope and role are the main elements)
    { @"entries.0.categories.0.term", kGDataCategoryACL },
    { @"entries.0.identifier", @"http://www.google.com/calendar/feeds/test%40gmail.com/acl/full/user%3Atest%40gmail.com" },
    { @"entries.0.content", @"" },
    { @"entries.0.links.1.rel", @"edit" },
    { @"entries.0.scope.type", @"user" },
    { @"entries.0.scope.value", @"test@gmail.com" },
    { @"entries.0.role.value", @"http://schemas.google.com/gCal/2005#owner" },
      
    { @"entries.0.unknownAttributes.@count", @"0" },
    { @"entries.0.unknownChildren.@count", @"0" },

      
    { @"", @"" }, // end of feed
      
    { nil, nil } // end of test array
  };
  
  [self runTests:tests];
}

@end
